#! /usr/bin/perl -w
#
# Priority logging for Snort
# 040708
# --------------------------------------------------------------------------
# Copyright: Gastinger, Gierlinger, Pascher Dec 2005
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL'.
# --------------------------------------------------------------------------

use POSIX qw(setsid);
use Getopt::Std;
use File::Copy "cp";
use Mail::Send;

$configfile = "/etc/snort/snortlogging.conf";
$infourl = "http://www.snort.org/pub-bin/sigs.cgi";

# CSV Log File Format
#timestamp,sig_generator,sig_id,sig_rev,msg,proto,src,srcport,dst,dstport,ethsrc,ethdst,ethlen,
#tcpflags,tcpseq,tcpack,tcpln,tcpwindow,ttl,tos,id,dgmlen,iplen,icmptype,icmpcode,icmpid,icmpseq,priority

# mögliche Kommandozeilen Optionen -h -s -d -p priority -m mailpriority -i inputfilename -o outputfilename
$opt_string = "hsdp:m:t:i:o:"; 

#tempory filename
$tempfile = "/tmp/snort.tmp";

# CLI Argumente
getopts("$opt_string",\my %opts);
usage() if ($opts{h});

# use Config File snortlogging.conf
if ( keys %opts == 0) {
	require $configfile;
	doParseLogging();
} else {
	#defaultparameter
	$priority = 2;
	# send mail to admin if priority <= $mailpriority 
	$mailpriority = 1;
	# default mail address
	$mailto = "root\@localhost";
	# default snortlogfilename
	$infile = "/var/log/snort/csv_snort.log";
	# default outputfilename
	$outfile = "/var/log/snort/snort.log";
	# run skript if filesize of $infile > $maxfilesize (in Bytes)
	$maxfilesize = 1000000;			
	$priority = $opts{p} if ($opts{p} && $opts{p} > 0 && $opts{p} < 6);
	$mailpriority = $opts{m} if ($opts{m} && $opts{m} > 0 && $opts{m} < 6);
	$mailto = $opts{t} if $opts{t};
	$infile = $opts{i} if $opts{i};
	$outfile = $opts{o} if $opts{o};
}

# daemon mode via CLI arguments
if ($opts{d}) {
	daemonize();
  	while (1==1) {
	  	if (-s $infile > $maxfilesize) {
	  		doParseLogging();
	  	}
	  	# sleep 5 minutes
	  	sleep(300);
	}		
}

sub doParseLogging {
	my @attacklines;
	my @logfilelinearray;
	my $mailsent = 0;
# copy logfile to a temporary file
	cp($infile,$tempfile) || die("copy failed");

# clear logfile
	open(DLOG,"> $infile") || die("File not found");close(DLOG);

# open prioritized to append data
	open(OUTPUTLOG,">> $outfile")  || die ("File not found");

# open temporary file to read data
	open(LOG,"$tempfile") || die("File not found");
	while (<LOG>) {
          chomp;
          @logfilelinearray = split(/,/);
          if ($logfilelinearray[-1] <= $mailpriority && $logfilelinearray[-1] > 0) {
        	$mailsent++;
		push(@attacklines,"$logfilelinearray[0]: $logfilelinearray[4] from $logfilelinearray[6]\n");
		if ( $logfilelinearray[1] != "" && $logfilelinearray[2] != "") {
		  push(@attacklines,"additional informations: $infourl?sid=$logfilelinearray[1]%3A$logfilelinearray[2]\n\n");
		}
          }
          if ($logfilelinearray[-1] <= $priority && $logfilelinearray[-1] > 0) {
# append prioritized logfile content to outputlog
                print OUTPUTLOG;
                print OUTPUTLOG "\n";
          }
	}
# if there are more than 1 alert -> send a second mail
	if ($mailsent > 0) {
		push(@attacklines,"\nThis mail was sent by Snort at Gibraltar Firewall\n");
        	sendMail(@attacklines);
	}
	
# close open files
	close(LOG);
	close(OUTPUTLOG);

# delete tmpfile
	unlink($tempfile);
}

sub sendMail {
	my(@mailbody)=@_;
    $msg = new Mail::Send;
    $msg = new Mail::Send Subject=>'Snort-Alert', To=>"$mailto";
    $msg->to("$mailto");
#   $msg->subject($mailsubject eq "" ? 'Gibraltar Security Alert - Snort':$mailsubject);
    $msg->subject("Gibraltar Security Alert - Snort");
#   $msg->set($header, @values);
    my $fh = $msg->open;               # some default mailer
    print $fh @mailbody;
#    print @mailbody;
    $fh->close;         # complete the message and send it
}


sub daemonize {
  chdir '/'                 or die "Can't chdir to /: $!";
  umask 0;
  open STDIN, '/dev/null'   or die "Can't read /dev/null: $!";
  open STDOUT, '>/dev/null' or die "Can't write to /dev/null: $!";
  open STDERR, '>/dev/null' or die "Can't write to /dev/null: $!";
  defined(my $pid = fork)   or die "Can't fork: $!";
  exit if $pid;
  setsid                    or die "Can't start a new session: $!";
}

sub usage {
  print STDERR << "EOF";
usage: $0 [-hdsv] [-p priority] [-m mailpriority] [-t emailadr] [-i file] [-o file]
  
-h          : show this help screen
-d          : run as daemon process
-s          : run as standard process
-p priority : define priority (highest 1 - lowest 5)
-m mailpriority: admin gets mail, if priority <= mailpriority
-t emailadr : admin email address
-i file     : input filename (snort cvs logfile)
-o file     : prioritized output filename

without cli-arguments, configfile $configfile will be used
 
example: $0 -d -p 2 -i /var/log/cvs_snort.log

outputfileformat: timestamp,sig_generator,sig_id,sig_rev,msg,proto,src,srcport,dst,
                  dstport,ethsrc,ethdst,ethlen,tcpflags,tcpseq,tcpack,tcpln,tcpwindow,
                  ttl,tos,id,dgmlen,iplen,icmptype,icmpcode,icmpid,icmpseq,priority

EOF
exit;
}
