#  Copyright (C) 2002  Stanislav Sinyagin
#  Adapted by Rene Mayrhofer for the file collector plugin
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



# Use a Perl plugin module as the collector source

package Torrus::Collector::File;

use Torrus::ConfigTree;
use Torrus::Log;

use strict;

# Register the collector type
$Torrus::Collector::collectorTypes{'file'} = 1;

###  Initialize the configuration validator with module-specific parameters

# optional parameter: 'file-regex'
# what those parameter do is the following: if a file-regex is defined, then
# search until the first line that matches this regex. this is the offset to
# the file-line field. if there is no file-regex, file-line will start from 
# the beginning of the file (offset=0). then file number file-line is read
# (counting from 0) and in that line, field number file-field is fetched
# (also counting from 0), whereby the fields are delimited by file-separator.
# trailing and leading null fields are stripped, i.e. delimiter characters in
# front or at the end of a line are ignored and the field number 0 therefore
# starts after any delimiter characters. file-separator can be a full regex,
# e.g. "\s" for the most common case of whitespace as delimiter characters.
my %validatorLeafParams =
    (
     'file-source'          => undef,
     'file-line'            => 0,
     'file-field'           => 0,
     'file-separator'       => '\s',
     'file-regex'           => '',
     );

sub initValidatorLeafParams
{
    my $hashref = shift;
    $hashref->{'ds-type'}{'collector'}{'collector-type'}{'file'} =
        \%validatorLeafParams;
}


$Torrus::Collector::params{'file'} = \%validatorLeafParams;

# expand our parameters where it makes sense
$Torrus::ConfigTree::expand_params{'file-source'} = 1;
$Torrus::ConfigTree::expand_params{'file-regex'} = 1;

$Torrus::Collector::initTarget{'file'} = \&Torrus::Collector::File::initTarget;

# This is obviously called once for each token.
sub initTarget
{
    my $collector = shift;
    my $token = shift;
    
    my $cref = $collector->collectorData( 'file' );
    
    # Create a list of all files that we need to read at each collector run
    # and, is there are line numbers for all occurances where the file is
    # referenced, remember the maximum line number we need to read (for
    # performance and memory consumption reasons).
    # TODO: optimization with line number
    my $source=$collector->param( $token, 'file-source' );
    $cref->{'byfile'}{$source}{$token} = 1;
    #my $linenum=$collector->param( $token, 'line' );
    #$cref->{'maxline'}{$source} = 1;
    
    return 1;
}


# This is first executed per target

$Torrus::Collector::runCollector{'file'} =
    \&Torrus::Collector::File::runCollector;

sub runCollector
{
    my $collector = shift;
    my $cref = shift;

    # For performance reasons, only read each file once, even if multiple 
    # tokens refer to it. This means to keep the files in memory until the
    # collector run has been completed.
    
    while ( my ($file, $tokens) = each %{$cref->{'byfile'}} )
    {
        Debug('Reading file '.$file);
        if (!open(FILE, $file)) {
            Warn("Error opening file $file from file collector, ignoring all tokens for this file.");
            close(FILE);
        }
        else {
            # 1. read file
            my @lines = <FILE>;
            close(FILE) || Warn("Error closing file $file. If this is a pipa, a zombie process might be left.");
            # remember when we finished reading the file and use that as the
            # time stamp
            my $now = time();
            
            # 2. extract all tokens that use this file
            foreach my $token ( keys %{$tokens} )
            {
                #Debug('Extracting token '.$token);
                my $line=$collector->param( $token, 'file-line' );
                my $field=$collector->param( $token, 'file-field' );
                my $regex=$collector->param( $token, 'file-regex' );
                my $separator=$collector->param( $token, 'file-separator' );
                #Debug("Reading field $field in line $line after regex '$regex' using separator '$separator'");
                
                my $offset=0;
                # if we have a regex, search until the first match
                if (defined $regex) {
                    while ($offset < scalar(@lines) && $lines[$offset] !~ /$regex/) {
                        $offset++;
                    }
                    if ($offset == scalar(@lines)) {
                        Warn("Warning: pattern '$regex' not found in file $file, ignoring token");
                    }
                }
                
                # now simply extract the field
                if ($offset+$line < scalar(@lines)) {
                    my $tmp=$lines[$offset+$line];
                    ##Debug ("+++ my line is '$tmp'");
                    
                    # ugly hack for those cases where the first field is empty
                    $tmp =~ s/^$separator*(.*?)$separator*$/$1/;
                    
                    my @fields=split(/$separator+/, $tmp);
                    
                    ##Debug ("+++ my fields are '@fields' --> ".scalar(@fields));
                    if ($field < scalar(@fields)) {
                        my $value=$fields[$field];
                        # strip potentially trailing newline characters
                        chomp $value;
                        # and also leading and trailing white space (the 
                        # separator can be somthing different from whitespace
                        # but we still have not use for that in the number)
                        $value =~ s/^\s+|\s+$//g;
                        # sanity check
                        if ($value =~ /^-?\d+\.?\d*$/) {
                            #Debug("Setting value: $value for token: $token");
                            $collector->setValue( $token, $value, $now );
                        }
                        else {
                            Warn("Warning: value '$value' is not a valid number, ignoring.");
                        }
                    }
                    else {
                        Warn("Warning: can not access field $field in line '$line' using separator '$separator'. End of line reached.");
                    }
                }
                else {
                    Warn("Warning: can not access line $line (+offset $offset) in file $file. End of file reached.");
                }
            }
        }
    }
}


1;


# Local Variables:
# mode: perl
# indent-tabs-mode: nil
# perl-indent-level: 4
# End:
